@@ -621,7 +621,7 @@ sinus_random_update (GObject   *unused,
 /* The note book                         */
 /*****************************************/
 
-gint
+static gint
 sinus_dialog (void)
 {
   GtkWidget *dlg;
@@ -811,7 +811,7 @@ sinus_dialog (void)
       gtk_box_pack_start(GTK_BOX(page), frame, FALSE, FALSE, 0);
       gtk_widget_show (frame);
 
-      vbox = GTK_BIN (frame)->child;
+      vbox = gtk_bin_get_child (GTK_BIN (frame));
 
       hbox = gtk_hbox_new (FALSE, 12);
       gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -948,7 +948,7 @@ sinus_dialog (void)
 /* Draw preview image. if DoCompute is TRUE then recompute image. */
 /******************************************************************/
 
-void
+static void
 sinus_do_preview (GtkWidget *widget)
 {
   static GtkWidget *theWidget = NULL;
