@@ -1,4 +1,4 @@
-/* -*- Mode: C; c-basic-offset: 4 -*- 
+/* -*- Mode: C; c-basic-offset: 4 -*-
     Gimp-Python - allows the writing of Gimp plugins in Python.
     Copyright (C) 1997-2002  James Henstridge <james@daa.com.au>
 
@@ -20,9 +20,19 @@
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
+
+#define NO_IMPORT_PYGOBJECT
+#include <pygobject.h>
+
 #include "pygimp.h"
+
+#define NO_IMPORT_PYGIMPCOLOR
+#include "pygimpcolor-api.h"
+
 #include <structmember.h>
 
+#include <glib-object.h>
+
 #ifndef PG_DEBUG
 # define PG_DEBUG 0
 #endif
@@ -66,41 +76,41 @@ pygimp_param_print(int nparams, GimpParam *params)
     for (i = 0; i < nparams; i++) {
 	switch(params[i].type) {
 	case GIMP_PDB_INT32:
-	    fprintf(stderr, "%i. int %i\n", i,
-		    params[i].data.d_int32);
+	    g_printerr("%i. int %i\n", i,
+                       params[i].data.d_int32);
 	    break;
 	case GIMP_PDB_INT16:
-	    fprintf(stderr, "%i. int %i\n", i,
-		    params[i].data.d_int16);
+	    g_printerr("%i. int %i\n", i,
+                       params[i].data.d_int16);
 	    break;
 	case GIMP_PDB_INT8:
-	    fprintf(stderr, "%i. int %i\n", i,
-		    params[i].data.d_int8);
+	    g_printerr("%i. int %u\n", i,
+                       params[i].data.d_int8);
 	    break;
 	case GIMP_PDB_FLOAT:
-	    fprintf(stderr, "%i. float %f\n", i,
-		    params[i].data.d_float);
+	    g_printerr("%i. float %f\n", i,
+                       params[i].data.d_float);
 	    break;
 	case GIMP_PDB_STRING:
-	    fprintf(stderr, "%i. string %s\n", i,
-		    params[i].data.d_string);
+	    g_printerr("%i. string %s\n", i,
+                       params[i].data.d_string);
 	    break;
 	case GIMP_PDB_INT32ARRAY:
 	case GIMP_PDB_INT16ARRAY:
 	case GIMP_PDB_INT8ARRAY:
 	case GIMP_PDB_FLOATARRAY:
 	case GIMP_PDB_STRINGARRAY:
-	    fprintf(stderr, "%i. array of type %i %s\n", i,
-		    params[i].type, params[i].data.d_int32array
-		    == NULL ? "(null)":"");
+	    g_printerr("%i. array of type %i %s\n", i,
+                       params[i].type,
+                       params[i].data.d_int32array == NULL ? "(null)":"");
 	    break;
 	case GIMP_PDB_STATUS:
-	    fprintf(stderr, "%i. status %i\n", i,
-		    params[i].data.d_status);
+	    g_printerr("%i. status %i\n", i,
+                       params[i].data.d_status);
 	    break;
 	default:
-	    fprintf(stderr, "%i. other %i\n", i,
-		    params[i].data.d_int32);
+	    g_printerr("%i. other %i\n", i,
+                       params[i].data.d_int32);
 	    break;
 	}
     }
@@ -250,11 +260,7 @@ pygimp_param_to_tuple(int nparams, const GimpParam *params)
 			PyString_FromString(""));
 	    break;
 	case GIMP_PDB_COLOR:
-	    {
-		guchar r, g, b;
-		gimp_rgb_get_uchar(&params[i].data.d_color, &r, &g, &b);
-		value = Py_BuildValue("(iii)", (int)r, (int)g, (int)b);
-	    }
+	    value = pygimp_rgb_new(&params[i].data.d_color);
 	    break;
 	case GIMP_PDB_REGION:
 	    value = Py_BuildValue("(iiii)",
@@ -281,11 +287,29 @@ pygimp_param_to_tuple(int nparams, const GimpParam *params)
 	case GIMP_PDB_SELECTION:
 	    value = pygimp_layer_new(params[i].data.d_selection);
 	    break;
-	case GIMP_PDB_BOUNDARY:
-	    value = PyInt_FromLong(params[i].data.d_boundary);
+	case GIMP_PDB_COLORARRAY:
+	    if (params[i].data.d_colorarray == NULL) {
+		value = PyTuple_New(0);
+		break;
+	    }
+	    if ((tmp=PyTuple_GetItem(args, i-1)) == NULL) {
+		Py_DECREF(args);
+		return NULL;
+	    }
+	    if (!PyInt_Check(tmp)) {
+		PyErr_SetString(PyExc_TypeError,
+				"count type must be integer");
+		Py_DECREF(args);
+		return NULL;
+	    }
+	    n = PyInt_AsLong(tmp);
+	    value = PyTuple_New(n);
+	    for (j = 0; j < n; j++)
+		PyTuple_SetItem(value, j,
+                                pygimp_rgb_new(&params[i].data.d_colorarray[j]));
 	    break;
-	case GIMP_PDB_PATH:
-	    value = PyInt_FromLong(params[i].data.d_path);
+	case GIMP_PDB_VECTORS:
+	    value = pygimp_vectors_new(params[i].data.d_vectors);
 	    break;
 	case GIMP_PDB_PARASITE:
 	    value = pygimp_parasite_new(gimp_parasite_copy(
@@ -305,10 +329,10 @@ pygimp_param_to_tuple(int nparams, const GimpParam *params)
 GimpParam *
 pygimp_param_from_tuple(PyObject *args, const GimpParamDef *ptype, int nparams)
 {
-    PyObject *tuple, *item, *r, *g, *b, *x, *y, *w, *h;
+    PyObject *tuple, *item, *x, *y, *w, *h;
     GimpParam *ret;
     int i, j, len;
-    gint32 *i32a; gint16 *i16a; gint8 *i8a; gdouble *fa; gchar **sa;
+    gint32 *i32a; gint16 *i16a; guint8 *i8a; gdouble *fa; gchar **sa;
 
     if (nparams == 0)
 	tuple = PyTuple_New(0);
@@ -320,11 +344,13 @@ pygimp_param_from_tuple(PyObject *args, const GimpParamDef *ptype, int nparams)
     }
     if (!PyTuple_Check(tuple)) {
 	PyErr_SetString(PyExc_TypeError, "wrong type of parameter");
+        Py_DECREF(tuple);
 	return NULL;
     }
 
     if (PyTuple_Size(tuple) != nparams) {
 	PyErr_SetString(PyExc_TypeError, "wrong number of parameters");
+        Py_DECREF(tuple);
 	return NULL;
     }
 
@@ -349,17 +375,17 @@ pygimp_param_from_tuple(PyObject *args, const GimpParamDef *ptype, int nparams)
 	switch (ptype[i-1].type) {
 	case GIMP_PDB_INT32:
 	    check((x = PyNumber_Int(item)) == NULL);
-	    ret[i].data.d_int32 = PyInt_AsLong(x);
+	    ret[i].data.d_int32 = (gint32)PyInt_AsLong(x);
 	    Py_DECREF(x);
 	    break;
 	case GIMP_PDB_INT16:
 	    check((x = PyNumber_Int(item)) == NULL);
-	    ret[i].data.d_int16 = PyInt_AsLong(x);
+	    ret[i].data.d_int16 = (gint16)PyInt_AsLong(x);
 	    Py_DECREF(x);
 	    break;
 	case GIMP_PDB_INT8:
 	    check((x = PyNumber_Int(item)) == NULL);
-	    ret[i].data.d_int8 = PyInt_AsLong(x);
+	    ret[i].data.d_int8 = (guint8)PyInt_AsLong(x);
 	    Py_DECREF(x);
 	    break;
 	case GIMP_PDB_FLOAT:
@@ -405,7 +431,7 @@ pygimp_param_from_tuple(PyObject *args, const GimpParamDef *ptype, int nparams)
 	case GIMP_PDB_INT8ARRAY:
 	    check(!PySequence_Check(item));
 	    len = PySequence_Length(item);
-	    i8a = g_new(gint8, len);
+	    i8a = g_new(guint8, len);
 	    for (j = 0; j < len; j++) {
 		x = PySequence_GetItem(item, j);
 		arraycheck((y=PyNumber_Int(x))==NULL,
@@ -446,14 +472,17 @@ pygimp_param_from_tuple(PyObject *args, const GimpParamDef *ptype, int nparams)
 	    ret[i].data.d_stringarray = sa;
 	    break;
 	case GIMP_PDB_COLOR:
-	    check(!PySequence_Check(item) || PySequence_Length(item) < 3);
-	    r = PySequence_GetItem(item, 0);
-	    g = PySequence_GetItem(item, 1);
-	    b = PySequence_GetItem(item, 2);
-	    check(!PyInt_Check(r) || !PyInt_Check(g) ||
-		  !PyInt_Check(b));
-	    gimp_rgb_set_uchar(&ret[i].data.d_color, PyInt_AsLong(r),
-			       PyInt_AsLong(g), PyInt_AsLong(b));
+	    {
+                GimpRGB rgb;
+
+                if (!pygimp_rgb_from_pyobject(item, &rgb)) {
+                    Py_DECREF(tuple);
+                    gimp_destroy_params(ret, nparams);
+                    return NULL;
+		}
+
+                ret[i].data.d_color = rgb;
+	    }
 	    break;
 	case GIMP_PDB_REGION:
 	    check(!PySequence_Check(item) ||
@@ -470,8 +499,12 @@ pygimp_param_from_tuple(PyObject *args, const GimpParamDef *ptype, int nparams)
 	    ret[i].data.d_region.height = PyInt_AsLong(h);
 	    break;
 	case GIMP_PDB_DISPLAY:
+            if (item == Py_None) {
+                ret[i].data.d_display = -1;
+                break;
+            }
 	    check(!pygimp_display_check(item));
-	    ret[i].data.d_display=((PyGimpDisplay *)item)->ID;
+	    ret[i].data.d_display = ((PyGimpDisplay *)item)->ID;
 	    break;
 	case GIMP_PDB_IMAGE:
 	    if (item == Py_None) {
@@ -479,7 +512,7 @@ pygimp_param_from_tuple(PyObject *args, const GimpParamDef *ptype, int nparams)
 		break;
 	    }
 	    check(!pygimp_image_check(item));
-	    ret[i].data.d_image=((PyGimpImage *)item)->ID;
+	    ret[i].data.d_image = ((PyGimpImage *)item)->ID;
 	    break;
 	case GIMP_PDB_LAYER:
 	    if (item == Py_None) {
@@ -487,7 +520,7 @@ pygimp_param_from_tuple(PyObject *args, const GimpParamDef *ptype, int nparams)
 		break;
 	    }
 	    check(!pygimp_layer_check(item));
-	    ret[i].data.d_layer=((PyGimpLayer *)item)->ID;
+	    ret[i].data.d_layer = ((PyGimpLayer *)item)->ID;
 	    break;
 	case GIMP_PDB_CHANNEL:
 	    if (item == Py_None) {
@@ -495,7 +528,7 @@ pygimp_param_from_tuple(PyObject *args, const GimpParamDef *ptype, int nparams)
 		break;
 	    }
 	    check(!pygimp_channel_check(item));
-	    ret[i].data.d_channel=((PyGimpChannel *)item)->ID;
+	    ret[i].data.d_channel = ((PyGimpChannel *)item)->ID;
 	    break;
 	case GIMP_PDB_DRAWABLE:
 	    if (item == Py_None) {
@@ -503,19 +536,33 @@ pygimp_param_from_tuple(PyObject *args, const GimpParamDef *ptype, int nparams)
 		break;
 	    }
 	    check(!pygimp_drawable_check(item));
-	    ret[i].data.d_channel=((PyGimpDrawable *)item)->ID;
+	    ret[i].data.d_channel = ((PyGimpDrawable *)item)->ID;
 	    break;
 	case GIMP_PDB_SELECTION:
 	    check(!pygimp_layer_check(item));
-	    ret[i].data.d_selection=((PyGimpLayer *)item)->ID;
+	    ret[i].data.d_selection = ((PyGimpLayer *)item)->ID;
 	    break;
-	case GIMP_PDB_BOUNDARY:
-	    check(!PyInt_Check(item));
-	    ret[i].data.d_boundary = PyInt_AsLong(item);
+	case GIMP_PDB_COLORARRAY:
+	    {
+                GimpRGB *rgb;
+
+		check(!PySequence_Check(item));
+		len = PySequence_Length(item);
+		rgb = g_new(GimpRGB, len);
+		for (j = 0; j < len; j++) {
+		    if (!pygimp_rgb_from_pyobject(item, &rgb[j])) {
+			Py_DECREF(tuple);
+			g_free(rgb);
+			gimp_destroy_params(ret, nparams);
+			return NULL;
+		    }
+		}
+                ret[i].data.d_colorarray = rgb;
+	    }
 	    break;
-	case GIMP_PDB_PATH:
-	    check(!PyInt_Check(item));
-	    ret[i].data.d_path = PyInt_AsLong(item);
+	case GIMP_PDB_VECTORS:
+	    check(!pygimp_vectors_check(item));
+	    ret[i].data.d_vectors = ((PyGimpVectors *)item)->ID;
 	    break;
 	case GIMP_PDB_PARASITE:
 	    /* can't do anything, since size of GimpParasite is not known */
@@ -545,14 +592,20 @@ pdb_query(PyGimpPDB *self, PyObject *args)
     int num, i;
     char **names;
     PyObject *ret;
+
     if (!PyArg_ParseTuple(args, "|zzzzzzz:gimp.pdb.query", &n, &b, &h, &a,
 			  &c, &d, &t))
 	return NULL;
+
     gimp_procedural_db_query(n, b, h, a, c, d, t, &num, &names);
+
     ret = PyList_New(num);
+
     for (i = 0; i < num; i++)
 	PyList_SetItem(ret, i, PyString_FromString(names[i]));
+
     g_free(names);
+
     return ret;
 }
 
@@ -567,11 +620,11 @@ static PyMethodDef pdb_methods[] = {
 PyObject *
 pygimp_pdb_new(void)
 {
-    PyGimpPDB *self;
-	
-    self = PyObject_NEW(PyGimpPDB, &PyGimpPDB_Type);
+    PyGimpPDB *self = PyObject_NEW(PyGimpPDB, &PyGimpPDB_Type);
+
     if (self == NULL)
 	return NULL;
+
     return (PyObject *)self;
 }
 
@@ -599,11 +652,14 @@ pdb_subscript(PyGimpPDB *self, PyObject *key)
 	PyErr_SetString(PyExc_TypeError, "Subscript must be a string");
 	return NULL;
     }
+
     r = (PyObject *)pygimp_pdb_function_new_from_proc_db(PyString_AsString(key));
+
     if (r == NULL) {
 	PyErr_Clear();
 	PyErr_SetObject(PyExc_KeyError, key);
     }
+
     return r;
 }
 
@@ -615,16 +671,60 @@ static PyMappingMethods pdb_as_mapping = {
 
 /* -------------------------------------------------------- */
 
+static PyObject *
+build_procedure_list(void)
+{
+    int num, i;
+    char **names, *name, *p;
+    PyObject *ret;
+
+    gimp_procedural_db_query(".*", ".*", ".*", ".*", ".*", ".*", ".*",
+                             &num, &names);
+
+    ret = PyList_New(num);
+
+    for (i = 0; i < num; i++) {
+        name = g_strdup(names[i]);
+        for (p = name; *p != '\0'; p++) {
+            if (*p == '-')
+                *p = '_';
+	}
+        PyList_SetItem(ret, i, PyString_FromString(name));
+        g_free(name);
+    }
+
+    g_free(names);
+
+    return ret;
+}
+
 static PyObject *
 pdb_getattro(PyGimpPDB *self, PyObject *attr)
 {
+    char *attr_name;
     PyObject *ret;
 
+    attr_name = PyString_AsString(attr);
+    if (!attr_name) {
+         PyErr_Clear();
+         return PyObject_GenericGetAttr((PyObject *)self, attr);
+    }
+
+    if (attr_name[0] == '_') {
+        if (!strcmp(attr_name, "__members__")) {
+            return build_procedure_list();
+        } else {
+            return PyObject_GenericGetAttr((PyObject *)self, attr);
+        }
+    }
+
     ret = PyObject_GenericGetAttr((PyObject *)self, attr);
     if (ret)
-	return ret;
+        return ret;
+
     PyErr_Clear();
-    return pygimp_pdb_function_new_from_proc_db(PyString_AsString(attr));
+
+    return pygimp_pdb_function_new_from_proc_db(attr_name);
 }
 
 PyTypeObject PyGimpPDB_Type = {
@@ -700,6 +800,7 @@ static void
 pf_dealloc(PyGimpPDBFunction *self)
 {
     g_free(self->name);
+
     Py_DECREF(self->proc_name);
     Py_DECREF(self->proc_blurb);
     Py_DECREF(self->proc_help);
@@ -709,8 +810,10 @@ pf_dealloc(PyGimpPDBFunction *self)
     Py_DECREF(self->proc_type);
     Py_DECREF(self->py_params);
     Py_DECREF(self->py_return_vals);
+
     gimp_destroy_paramdefs(self->params, self->nparams);
     gimp_destroy_paramdefs(self->return_vals, self->nreturn_vals);
+
     PyObject_DEL(self);
 }
 
@@ -744,89 +847,142 @@ pf_call(PyGimpPDBFunction *self, PyObject *args, PyObject *kwargs)
     GimpParam *params, *ret;
     int nret;
     PyObject *t = NULL, *r;
+    GimpRunMode run_mode = GIMP_RUN_NONINTERACTIVE;
 
 #if PG_DEBUG > 0
-    fprintf(stderr, "--- %s --- ", PyString_AsString(self->proc_name));
+    g_printerr("--- %s --- ", PyString_AsString(self->proc_name));
 #endif
-    if (self->nparams > 0 && !strcmp(self->params[0].name, "run_mode")) {
-	params = pygimp_param_from_tuple(args, self->params+1, self->nparams-1);
+
+    if (kwargs) {
+        Py_ssize_t len, pos;
+        PyObject *key, *val;
+
+        len = PyDict_Size(kwargs);
+
+        if (len == 1) {
+            pos = 0;
+            PyDict_Next(kwargs, &pos, &key, &val);
+
+            if (!PyString_Check(key)) {
+                PyErr_SetString(PyExc_TypeError,
+                                "keyword argument name is not a string");
+                return NULL;
+            }
+
+            if (strcmp(PyString_AsString(key), "run_mode") != 0) {
+                PyErr_SetString(PyExc_TypeError,
+                                "only 'run_mode' keyword argument accepted");
+                return NULL;
+            }
+
+            if (pyg_enum_get_value(GIMP_TYPE_RUN_MODE, val, (gpointer)&run_mode))
+                return NULL;
+        } else if (len != 0) {
+            PyErr_SetString(PyExc_TypeError,
+                            "expecting at most one keyword argument");
+            return NULL;
+        }
+    }
+
+    if (self->nparams > 0 && !strcmp(self->params[0].name, "run-mode")) {
+	params = pygimp_param_from_tuple(args, self->params + 1,
+					 self->nparams - 1);
+
 	if (params == NULL)
 	    return NULL;
+
 	params[0].type = self->params[0].type;
-	params[0].data.d_int32 = GIMP_RUN_NONINTERACTIVE;
+	params[0].data.d_int32 = run_mode;
+
 #if PG_DEBUG > 1
 	pygimp_param_print(self->nparams, params);
 #endif
-	ret = gimp_run_procedure2(self->name, &nret, self->nparams,
-				  params);
+
+	ret = gimp_run_procedure2(self->name, &nret, self->nparams, params);
     } else {
 	params = pygimp_param_from_tuple(args, self->params, self->nparams);
+
 	if (params == NULL)
 	    return NULL;
+
 #if PG_DEBUG > 1
 	pygimp_param_print(self->nparams, params+1);
 #endif
-	ret = gimp_run_procedure2(self->name, &nret, self->nparams,
-				  params+1);
+
+	ret = gimp_run_procedure2(self->name, &nret, self->nparams, params + 1);
     }
+
     gimp_destroy_params(params, self->nparams);
+
     if (!ret) {
 	PyErr_SetString(pygimp_error, "no status returned");
 #if PG_DEBUG >= 1
-	fprintf(stderr, "ret == NULL\n");
+	g_printerr("ret == NULL\n");
 #endif
 	return NULL;
     }
+
     switch(ret[0].data.d_status) {
-    case GIMP_PDB_EXECUTION_ERROR:
-#if PG_DEBUG > 0
-	fprintf(stderr, "execution error\n");
-#endif
-	gimp_destroy_params(ret, nret);
-	PyErr_SetString(PyExc_RuntimeError, "execution error");
-	return NULL;
-	break;
-    case GIMP_PDB_CALLING_ERROR:
-#if PG_DEBUG > 0
-	fprintf(stderr, "calling error\n");
-#endif
-	gimp_destroy_params(ret, nret);
-	PyErr_SetString(PyExc_TypeError, "invalid arguments");
-	return NULL;
-	break;
     case GIMP_PDB_SUCCESS:
 #if PG_DEBUG > 0
-	fprintf(stderr, "success\n");
+	g_printerr("success\n");
 #endif
 	t = pygimp_param_to_tuple(nret-1, ret+1);
 	gimp_destroy_params(ret, nret);
+
 	if (t == NULL) {
-	    PyErr_SetString(pygimp_error,
-			    "couldn't make return value");
+	    PyErr_SetString(pygimp_error, "could not make return value");
 	    return NULL;
 	}
 	break;
+
+    case GIMP_PDB_EXECUTION_ERROR:
+#if PG_DEBUG > 0
+	g_printerr("execution error\n");
+#endif
+        PyErr_SetString(PyExc_RuntimeError, gimp_get_pdb_error());
+	gimp_destroy_params(ret, nret);
+	return NULL;
+
+    case GIMP_PDB_CALLING_ERROR:
+#if PG_DEBUG > 0
+	g_printerr("calling error\n");
+#endif
+        PyErr_SetString(PyExc_RuntimeError, gimp_get_pdb_error());
+	gimp_destroy_params(ret, nret);
+	return NULL;
+
+    case GIMP_PDB_CANCEL:
+#if PG_DEBUG > 0
+	g_printerr("cancel\n");
+#endif
+        PyErr_SetString(PyExc_RuntimeError, gimp_get_pdb_error());
+	gimp_destroy_params(ret, nret);
+	return NULL;
+
     default:
 #if PG_DEBUG > 0
-	fprintf(stderr, "unknown - %i (type %i)\n",
-		ret[0].data.d_status, ret[0].type);
+	g_printerr("unknown - %i (type %i)\n",
+                   ret[0].data.d_status, ret[0].type);
 #endif
 	PyErr_SetString(pygimp_error, "unknown return code");
 	return NULL;
-	break;
     }
+
     if (PyTuple_Size(t) == 1) {
 	r = PyTuple_GetItem(t, 0);
 	Py_INCREF(r);
 	Py_DECREF(t);
 	return r;
     }
+
     if (PyTuple_Size(t) == 0) {
 	r = Py_None;
 	Py_INCREF(r);
 	Py_DECREF(t);
 	return r;
     }
+
     return t;
 }
 
@@ -885,6 +1041,7 @@ pygimp_pdb_function_new(const char *name, const char *blurb, const char *help,
     int i;
 
     self = PyObject_NEW(PyGimpPDBFunction, &PyGimpPDBFunction_Type);
+
     if (self == NULL)
 	return NULL;
 
@@ -916,5 +1073,6 @@ pygimp_pdb_function_new(const char *name, const char *blurb, const char *help,
 				      return_vals[i].type,
 				      return_vals[i].name,
 				      return_vals[i].description));
+
     return (PyObject *)self;
 }
